import re
import types
import importlib.machinery
import pyrpm.spec

import redpesk_utils.common
from .error import EmptySpecfileError


def re_tag_compile(tag):
    return re.compile(tag, re.IGNORECASE)


tagsHack = [
    pyrpm.spec._NameValue("name", re_tag_compile(r"^Name:\s*(\S+)")),
    pyrpm.spec._NameValue("url", re_tag_compile(r"^URL:\s*(\S+)")),
    pyrpm.spec._NameValue("sha", re_tag_compile(rf"^{redpesk_utils.common.MACRO_HEXSHA}:\s*(\S+)")),
    pyrpm.spec._NameValue("version", re_tag_compile(r"^Version:\s*(\S+)")),
    pyrpm.spec._NameValue("release", re_tag_compile(r"^Release:\s*(\S+)")),
    pyrpm.spec._NameValue(
        "excludearch", re_tag_compile(r"^ExcludeArch:\s*(\S+)")),
    pyrpm.spec._ListAndDict(
        "sources",
        re_tag_compile(r"^(Source\d*):\s*(\S+)")),
    pyrpm.spec._MacroDef('define', re.compile(r'^%define\s+(\S+)\s+(\S+)')),
    pyrpm.spec._MacroDef('global', re.compile(r'^%global\s+(\S+)\s+(\S+)'))
]

pyrpm.spec._tags = tagsHack
Spec = pyrpm.spec.Spec
setattr(pyrpm.spec.Package, "sources", list())
pyrpm.spec._macro_pattern = "%{(\\S+?)(:\\S+?}*)*}"

def from_string(string: str):
    if not string:
        raise EmptySpecfileError("specfile is empty")
    spec = pyrpm.spec.Spec()
    parse_context = {"current_subpackage": None}
    for line in string.splitlines():
        spec, parse_context = _parse(spec, parse_context, line)
    return spec


def _parse(spec_obj, context, line):
    for tag in [tag for tag in pyrpm.spec._tags if not getattr(spec_obj, tag.name)]:
        match = tag.test(line)
        if match:
            return tag.update(spec_obj, context, match, line)
    return spec_obj, context

def load_source(modname, modpath):
    loader = importlib.machinery.SourceFileLoader(modname, modpath)
    mod = types.ModuleType(loader.name)
    loader.exec_module(mod)
    return mod
