import os
import re
import json
import typing

import redpesk_utils.type
import redpesk_utils.common
import redpesk_utils.parameter


class Gitpkg:
    @redpesk_utils.type.check_types
    def __init__(
            self,
            namespace: redpesk_utils.type.NonGroupSpecialText,
            gitpkg: redpesk_utils.type.NonSpecialText) -> None:
        self._namespace = namespace
        self._gitpkg = gitpkg
        self._specfile_name = f"{gitpkg}{redpesk_utils.common.SPEC_EXT}"
        self._specfile_autogen_path = os.path.join(
            redpesk_utils.common.REDPESK_FOLDER,
            f"{redpesk_utils.common.AUTOGEN_PREFIX}{self._specfile_name}")
        self._gitpkg_space = os.path.join(namespace, gitpkg)

    @property
    def namespace(self) -> str:
        return self._namespace

    @property
    def gitpkg(self) -> str:
        return self._gitpkg

    @property
    def specfile_name(self) -> str:
        return self._specfile_name

    @property
    def specfile_autogen_path(self) -> str:
        return self._specfile_autogen_path

    @property
    def gitpkg_space(self) -> str:
        return self._gitpkg_space


def autogen_archive_name(
        metadata: str,
        files: typing.List[str],
        gitpkg: Gitpkg) -> str:
    if redpesk_utils.parameter.get_key(
            json.loads(metadata),
            redpesk_utils.common.FIELD_SOURCES_URI):
        for file in files:
            if re.search(
                f"{re.escape(gitpkg.gitpkg)}-{redpesk_utils.common.REGEX_ARCHIVE_VERSION}",
                    file):
                return os.path.basename(file)
        return str()
