import re
import typing
import logging

import redpesk_utils.common


class LoggingFormatter(logging.Formatter):
    def __init__(self) -> None:
        super().__init__()
        self._formatter: typing.Dict[
            int,
            logging.Formatter] = {
            logging.INFO: logging.Formatter(
                f"{redpesk_utils.common.LOGGING_COLOUR[logging.INFO]}{redpesk_utils.common.LOGGING_FORMAT}{redpesk_utils.common.COLOUR_RESET}"),
            logging.DEBUG: logging.Formatter(
                f"{redpesk_utils.common.LOGGING_COLOUR[logging.DEBUG]}{redpesk_utils.common.LOGGING_FORMAT}{redpesk_utils.common.COLOUR_RESET}"),
            logging.WARNING: logging.Formatter(
                f"{redpesk_utils.common.LOGGING_COLOUR[logging.WARNING]}{redpesk_utils.common.LOGGING_FORMAT}{redpesk_utils.common.COLOUR_RESET}"),
            logging.ERROR: logging.Formatter(
                f"{redpesk_utils.common.LOGGING_COLOUR[logging.ERROR]}{redpesk_utils.common.LOGGING_FORMAT}{redpesk_utils.common.COLOUR_RESET}")}

    def format(self, record: logging.LogRecord) -> str:
        record.message = re.sub(
            redpesk_utils.common.REGEX_UUID,
            f"{redpesk_utils.common.COLOUR_BLUE}\\g<0>{redpesk_utils.common.LOGGING_COLOUR[record.levelno]}",
            record.getMessage())
        return self._formatter[record.levelno].formatMessage(record)


def format(*args: typing.Tuple[str]) -> str:
    return redpesk_utils.common.LOGGING_SEPARATOR.join(map(str, args))


def setHandler() -> logging.StreamHandler:
    handler = logging.StreamHandler()
    handler.setFormatter(LoggingFormatter())
    return handler


def getLogger(name: str, level=logging.INFO) -> logging.Logger:
    logger = logging.getLogger(name)
    logger.addHandler(red_handler)
    logger.propagate = False
    logger.setLevel(level)
    return logger


red_handler = setHandler()
red_log = getLogger(__package__)
