import re
import enum
import typing
import inspect

import redpesk_utils.error
import redpesk_utils.common


class Type(enum.Enum):
    int = redpesk_utils.common.TYPE_INT
    bool = redpesk_utils.common.TYPE_BOOL
    float = redpesk_utils.common.TYPE_FLOAT
    string = redpesk_utils.common.TYPE_STRING
    build_type = redpesk_utils.common.TYPE_BUILD_TYPE
    string_list = redpesk_utils.common.TYPE_STRING_LIST
    string_array = redpesk_utils.common.TYPE_STRING_ARRAY
    specfile_type = redpesk_utils.common.TYPE_SPECFILE_TYPE


class GType:
    def __init__(self) -> None:
        pass


class NonNullText(GType):
    def __init__(self, value: str) -> None:
        super().__init__()
        if not value:
            raise redpesk_utils.error.TypeError(value, self.__class__.__name__)


class NonSpecialText(NonNullText):
    def __init__(self, value: str) -> None:
        super().__init__(value)
        if not re.search(redpesk_utils.common.REGEX_NONSPECIALTEXT, value):
            raise redpesk_utils.error.TypeError(value, self.__class__.__name__)


class NonGroupSpecialText(NonNullText):
    def __init__(self, value: str) -> None:
        super().__init__(value)
        if not re.search(redpesk_utils.common.REGEX_NONGROUPTEXT,
                         value) or value[-1] == '/':
            raise redpesk_utils.error.TypeError(value, self.__class__.__name__)


class ShaText(NonNullText):
    def __init__(self, value: str) -> None:
        super().__init__(value)
        if not re.search(redpesk_utils.common.REGEX_SHA, value):
            raise redpesk_utils.error.TypeError(value, self.__class__.__name__)


def check_instance(
        key: str, parameter: inspect.Parameter, arg: str) -> None:
    if parameter.annotation is parameter.empty:
        return
    if GType in parameter.annotation.__mro__:
        try:
            parameter.annotation(arg)
        except ValueError as error:
            raise redpesk_utils.error.TypeError(key, arg, str(error))
    elif not isinstance(arg, parameter.annotation):
        raise redpesk_utils.error.TypeError(key, arg, parameter.annotation)


def check_types(func: typing.Callable) -> typing.Callable:
    def inner1(*args: typing.Tuple[typing.Any], **
               kwargs: typing.Dict[str, typing.Any]) -> typing.Any:
        parameters: typing.List(
            str, inspect.Parameter) = list(
            inspect.signature(func).parameters.items())
        for i in range(0, len(args)):
            (key, parameter) = parameters[i]
            check_instance(key, parameter, args[i])
        for (key, parameter) in parameters[len(args):]:
            kwarg: typing.Dict = kwargs.get(key, None)
            if kwarg or isinstance(kwarg, str):
                check_instance(key, parameter, kwarg)
        return func(*args, **kwargs)
    return inner1
